const state = require('../state');
const {createUIEvent, createFocusEvent} = require('../event');
const {focusable} = require('../util').element;

state.register('page', {
	focusElement: null,
	uploadingInputElement: null,
	currentElement: document.documentElement
});

exports.setFocusElement = function () {
	const {focusElement, currentElement} = state.get('page');

	if (focusElement === currentElement) {
		return;
	}
		
	state.update('page', clone => {
		const oldFocusElement = focusElement;

		oldFocusElement && oldFocusElement.dispatchEvent(createFocusEvent('focusout'));

		clone.focusElement = 
			currentElement && focusable(currentElement) ? currentElement: null;

		clone.focusElement && clone.focusElement.dispatchEvent(createFocusEvent('focusin'));
		oldFocusElement && oldFocusElement.dispatchEvent(createUIEvent('blur'));
		clone.focusElement && clone.focusElement.dispatchEvent(createUIEvent('focus'));
	});
};

exports.dispatchEvent = function (event) {
	if (!(event instanceof Event)) {
		throw new Error('Can not dispatch a non-event object');
	}
	
	state.get('page').currentElement.dispatchEvent(event);
	
	return true;
};